package com.jjt.biz.controller;

import com.jjt.biz.domain.TwinDevice;
import com.jjt.biz.service.ITwinDeviceService;
import com.jjt.biz.vo.IndexData;
import com.jjt.calc.domain.TwinCalcDay;
import com.jjt.calc.service.ITwinCalcAlarmsService;
import com.jjt.calc.service.ITwinCalcDayService;
import com.jjt.calc.service.ITwinCalcStopService;
import com.jjt.common.constant.CacheConstants;
import com.jjt.common.core.controller.BaseController;
import com.jjt.common.core.domain.R;
import com.jjt.common.core.redis.RedisCache;
import com.jjt.common.utils.DateUtils;
import com.jjt.utils.AsyncService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.ZoneOffset;
import java.util.*;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.Future;
import java.util.stream.Collectors;

/**
 * swagger 用户测试方法
 *
 * @author ruoyi
 */
@Api("数据接口")
@RestController
@Slf4j
public class ApiController extends BaseController {
    @Resource
    private ITwinCalcDayService twinCalcDayService;
    @Resource
    private ITwinCalcStopService stopService;
    @Resource
    private ITwinCalcAlarmsService alarmsService;
    @Resource
    private ITwinDeviceService deviceService;
    @Resource
    private AsyncService asyncService;
    @Resource
    private RedisCache redisCache;

    @ApiOperation("在线设备")
    @GetMapping("/api/device/online")
    @CrossOrigin(origins = "*")
    public R<List<String>> online() {
        TwinDevice searchDevice = new TwinDevice();
        searchDevice.setOnline("1");
        List<TwinDevice> list = deviceService.selectTwinDeviceList(searchDevice);
        List<String> devices = new ArrayList<>();
        list.forEach(d -> devices.add(d.getDeviceCode()));
        return R.success(devices);
    }

    @ApiOperation("首页统计数据")
    @GetMapping("/api/index")
    @CrossOrigin(origins = "*")
    public R<IndexData> index() {
        return R.success(redisCache.getCacheObject(CacheConstants.INDEX_CALC));
    }

    @ApiOperation("首页告警数据")
    @GetMapping("/api/alarm")
    @CrossOrigin(origins = "*")
    public R<IndexData> alarm() {
        return R.success(redisCache.getCacheObject(CacheConstants.INDEX_ALARM));
    }

    @ApiOperation("设备具体数据")
    @GetMapping("/api/query/{device}")
    @CrossOrigin(origins = "*")
    public R<Map> query(@PathVariable String device, String week) {
        TwinDevice info = deviceService.selectTwinDeviceByCode(device);
        if (info == null) {
            return R.fail("该设备未接入");
        }
        Future<Map<String, Object>> future = asyncService.currData(info);
        Map<String, Object> result = new HashMap<>(16);
        try {
            result.put("data", future.get());
        } catch (InterruptedException | ExecutionException e) {
            throw new RuntimeException(e);
        }
        if (week != null) {
            //7点前是属于前一生产天,所以这里直接减7小时即可
            LocalDateTime ldt = LocalDateTime.now().minusHours(7);
            //获取前面7天的数据
            LocalDate localDate = ldt.toLocalDate().minusDays(7);
            Date date = Date.from(localDate.atStartOfDay(ZoneOffset.of("+8")).toInstant());
            List<TwinCalcDay> list = twinCalcDayService.selectTwinCalcDayListByTime(date);
            List<Map<String, Object>> alarms = new ArrayList<>();
            Map<Date, List<TwinCalcDay>> stopDeviceGroup = list.stream().collect(Collectors.groupingBy(TwinCalcDay::getTime, LinkedHashMap::new, Collectors.toList()));
            for (Map.Entry<Date, List<TwinCalcDay>> entry : stopDeviceGroup.entrySet()) {
                Map<String, Object> temp = new HashMap<>(16);
                TwinCalcDay day = new TwinCalcDay(entry.getKey());
                List<TwinCalcDay> days = entry.getValue();
                day.calcDays(days);
                temp.put("time", DateUtils.parseDateToStr(entry.getKey()));
                temp.put("value", day.getAlarm());
                alarms.add(temp);
            }
            result.put("alarms", alarms);
        }
        return R.success(result);
    }
}
